#pragma once

class BaseMutex
{
public:
	BaseMutex() {
		InitializeCriticalSection(&_crit);
	}
	~BaseMutex() {
		DeleteCriticalSection(&_crit);
	}
	void lock() {
		EnterCriticalSection(&_crit);
	}
	void unlock() {
		LeaveCriticalSection(&_crit);
	}
private:
	CRITICAL_SECTION _crit;
	// no copy
	BaseMutex(const BaseMutex&);
	void operator=(const BaseMutex&);
};

class AutoLocker
{
public:
	AutoLocker(BaseMutex& mtx) :_mtx(mtx) {
		printf("lock now\n");
		_mtx.lock();
	}
	~AutoLocker() {
		printf("unlock now\n");
		_mtx.unlock();
	}
private:
	BaseMutex& _mtx;
	// no copy
	AutoLocker(const AutoLocker&);
	void operator=(const AutoLocker&);
};
typedef BaseMutex t_mutex;